''
''  QWICC OLED Driver
''

obj
  i2c : "Basic_I2C_Driver"


con
  QWIIC_OLED             =    $3D                                ' default I2C address for OLED panel

'' Definitions for SparkFun I2C OLED Breakout

  #0, BLACK, WHITE

  LCDWIDTH               =    64                                 ' pixel size of display
  LCDHEIGHT              =    48

  #0, NORM, XOR_DOT
  
  #0, PAGE, ALL

  #0, WIDGETSTYLE0, WIDGETSTYLE1, WIDGETSTYLE2

  I2C_COMMAND            =    $00
  I2C_DATA               =    $40

  SETCONTRAST            =    $81
  DISPLAYALLONRESUME     =    $A4
  DISPLAYALLON           =    $A5
  NORMALDISPLAY          =    $A6
  INVERTDISPLAY          =    $A7
  DISPLAYOFF             =    $AE
  DISPLAYON              =    $AF
  SETDISPLAYOFFSET       =    $D3
  SETCOMPINS             =    $DA
  SETVCOMDESELECT        =    $DB
  SETDISPLAYCLOCKDIV     =    $D5
  SETPRECHARGE           =    $D9
  SETMULTIPLEX           =    $A8
  SETLOWCOLUMN           =    $00
  SETHIGHCOLUMN          =    $10
  SETSTARTLINE           =    $40
  MEMORYMODE             =    $20
  COMSCANINC             =    $C0
  COMSCANDEC             =    $C8
  SEGREMAP               =    $A0
  CHARGEPUMP             =    $8D
  EXTERNALVCC            =    $01
  SWITCHCAPVCC           =    $02
  SETPAGENUMBER          =    $B0

'' Scroll

  ACTIVATESCROLL         =    $2F
  DEACTIVATESCROLL       =    $2E
  SETVERTICALSCROLLAREA  =    $A3
  RIGHTHORIZONTALSCROLL  =    $26
  LEFT_HORIZONTALSCROLL  =    $27
  VERTICALRIGHTHORIZONTALSCROLL  =   $29
  VERTICALLEFTHORIZONTALSCROLL   =   $2A

'' Recognized control characters

  HOME                   =    $01                                ' cursor to 0,0 and clear page
  NL                     =    $0A                                ' new line or '\n'
  CR                     =    $0D                                ' carriage return or '\r'
  SETX                   =    $10                                ' SETX+0..SETX+9 sets X cursor
  SETY                   =    $1A                                ' SETY+0..SETY+5 sets Y cursor


var
  byte  oled_scl                                                 ' I/O pin for SCL. SDA is one higher
  byte  oled_addr                                                ' I2C address for OLED controller
  byte  cursorX, cursorY                                         ' pixel cursor location
  byte  foreColor, drawMode                                      ' foreground color and draw mode
  byte  screenmemory[(LCDWIDTH * LCDHEIGHT) / 8]                 ' 8 pixels per byte

	{{ LCD Memory organised in 64 horizontal pixel and 6 rows of byte
	 B  B .............B  -----
	 y  y .............y        \
	 t  t .............t         \
	 e  e .............e          \
	 0  1 .............63          \
	                                \
	 D0 D0.............D0            \
	 D1 D1.............D1            / ROW 0
	 D2 D2.............D2           /
	 D3 D3.............D3          /
	 D4 D4.............D4         /
	 D5 D5.............D5        /
	 D6 D6.............D6       /
	 D7 D7.............D7  ----
	}}


pub init( scl_pin, addr ) : error
  oled_scl := scl_pin
  oled_addr := addr
  i2c.Initialize(oled_scl)                                       ' initialize I2C
  setColor(WHITE)
  setDrawMode(NORM)                                              ' note: power on will reset
  setCursor(0,0)                                                 '       SSD1306 controller

  ifnot error:=writeRegister(oled_addr, I2C_COMMAND, DISPLAYOFF) ' $AE
    writeRegister(oled_addr, I2C_COMMAND, SETDISPLAYCLOCKDIV)    ' $D5
    writeRegister(oled_addr, I2C_COMMAND, $80)                   ' use suggested ratio
    writeRegister(oled_addr, I2C_COMMAND, SETMULTIPLEX)          ' $A8
    writeRegister(oled_addr, I2C_COMMAND, $2F)
    writeRegister(oled_addr, I2C_COMMAND, SETDISPLAYOFFSET)      ' $D3
    writeRegister(oled_addr, I2C_COMMAND, 0)                     ' no offset
    writeRegister(oled_addr, I2C_COMMAND, SETSTARTLINE | $00)    ' line #0
    writeRegister(oled_addr, I2C_COMMAND, CHARGEPUMP)            ' enable charge pump
    writeRegister(oled_addr, I2C_COMMAND, $14)
    writeRegister(oled_addr, I2C_COMMAND, NORMALDISPLAY)         ' $A6
    writeRegister(oled_addr, I2C_COMMAND, DISPLAYALLONRESUME)    ' $A
    writeRegister(oled_addr, I2C_COMMAND, SEGREMAP | $01)
    writeRegister(oled_addr, I2C_COMMAND, COMSCANDEC)
    writeRegister(oled_addr, I2C_COMMAND, SETCOMPINS)            ' $DA
    writeRegister(oled_addr, I2C_COMMAND, $12)
    writeRegister(oled_addr, I2C_COMMAND, SETCONTRAST)           ' $81
    writeRegister(oled_addr, I2C_COMMAND, $8F)
    writeRegister(oled_addr, I2C_COMMAND, SETPRECHARGE)          ' $D9
    writeRegister(oled_addr, I2C_COMMAND, $F1)
    writeRegister(oled_addr, I2C_COMMAND, SETVCOMDESELECT)       ' $DB
    writeRegister(oled_addr, I2C_COMMAND, $40)
    writeRegister(oled_addr, I2C_COMMAND, DISPLAYON)             ' turn on oled panel
    clear(ALL) '' Erase hardware memory inside OLED controller to avoid random data in memory.

pub readRegister( addr, register, bytes) | t,ackbit,error,up     ' read 1-4 bytes starting at register
                                                                 ' if bytes < 0 then use MSB..LSB order
  i2c.start(oled_scl)                                            ' start a sequence
  error := i2c.write(oled_scl,(addr<<1)|i2c#Xmit) <> 0           ' write chip address ... write mode
  error |= i2c.write(oled_scl,register) <> 0                     ' write register address
  i2c.start(oled_scl)                                            ' start another sequence
  error |= i2c.write(oled_scl,(addr<<1)|i2c#Recv) <> 0           ' change to read mode
  if error                                                       ' return true if any unexpected NAKs
    return true
  up := (||bytes) - 1
  if bytes < 0                                                   ' read requested number of bytes
    repeat t from up to 0                                        ' use MSB..LSB order
      ackbit := ((t<>0)&i2c#ACK)|((t==0)&i2c#NAK)                ' ACK for all but the last, then NAK
      result := (result << 8) | i2c.read(oled_scl,ackbit)
  else 
    repeat t from 0 to up                                        ' use LSB..MSB order
      ackbit := ((t<>up)&i2c#ACK)|((t==up)&i2c#NAK)              ' ACK for all but the last, then NAK
      result |= i2c.read(oled_scl,ackbit) << (t<<3)              ' read bytes in LSB..MSB order
  i2c.stop(oled_scl)

pub writeRegister( addr, register, value) : error                ' write a byte to a register
  i2c.start(oled_scl)                                            ' start a sequence
  error := i2c.write(oled_scl,(addr<<1)|i2c#Xmit) <> 0           ' write chip address ... write mode
  error |= i2c.write(oled_scl,register) <> 0                     ' write register address
  error |= i2c.write(oled_scl,value) <> 0                        ' write byte value
  i2c.stop(oled_scl)                                             ' any unexpected NAK is an error

pub setPageAddress(addr)                                         ' set SSD1306 page address
  writeRegister(oled_addr, I2C_COMMAND, SETPAGENUMBER | addr)

pub setColumnAddress(addr)                                       ' set SSD1306 column address
  writeRegister(oled_addr, I2C_COMMAND, SETHIGHCOLUMN | (addr>>4) + $02)
  writeRegister(oled_addr, I2C_COMMAND, SETLOWCOLUMN | (addr&$0F))

pub clear(mode) | i                                              ' clear GDRAM (ALL) or page buffer (PAGE)
  if mode == ALL
    repeat i from 0 to 7
      setPageAddress(i)
      setColumnAddress(0)
      repeat 128
        writeRegister(oled_addr, I2C_DATA, 0)
  else
    bytefill(@screenmemory,0,384)

pub invert(inv)                                                  ' invert display BLACK <-> WHITE
  if inv
    writeRegister(oled_addr, I2C_COMMAND, INVERTDISPLAY)
  ELSE
    writeRegister(oled_addr, I2C_COMMAND, NORMALDISPLAY)

pub setColor(color)                                              ' set current draw color
  foreColor := color

pub setDrawMode(mode)                                            ' set current draw mode
  drawMode := mode

pub setCursor(x,y)                                               ' set current pixel position
  cursorX := x
  cursorY := y

pub contrast(contrastVal)                                        ' set display contrast (0..255)
  writeRegister(oled_addr, I2C_COMMAND, SETCONTRAST)             ' $81
  writeRegister(oled_addr, I2C_COMMAND, contrastVal)

pub updateDisplay | i, j                                         ' copy screen buffer to controller
  repeat i from 0 to 5
    setPageAddress(i)
    setColumnAddress(0)
      repeat j from 0 to 63
        writeRegister(oled_addr, I2C_DATA, screenmemory[i*64+j])

pub flipVertical(flip)
  if flip
    writeRegister(oled_addr, I2C_COMMAND, COMSCANINC)
  else
    writeRegister(oled_addr, I2C_COMMAND, COMSCANDEC)

pub flipHorizontal(flip)
  if flip
    writeRegister(oled_addr, I2C_COMMAND, SEGREMAP | $00)
  else
    writeRegister(oled_addr, I2C_COMMAND, SEGREMAP | $01)

pub pixel(x,y,color,mode) | t                                    ' draw a single pixel
  if (x < 0) OR (x => LCDWIDTH) OR (y < 0) OR (y => LCDHEIGHT)
    return
  t := |< (y & 7)                                                ' get bit for selected row
  if mode == XOR_DOT
    if color == WHITE
      screenmemory[x + (y >> 3) * LCDWIDTH] ^= t                 ' invert pixel
  else
    if color == WHITE
      screenmemory[x + (y >> 3) * LCDWIDTH] |= t                 ' turn pixel on
    else
      screenmemory[x + (y >> 3) * LCDWIDTH] &= !t                ' turn pixel off

pub drawChar(x,y,ch,color,mode) | temp, i, j, base               ' draw a character on the display
  base := (ch - fontStart) * fontWidth                           ' base of font for character
  repeat i from 0 to fontWidth
    if i == fontWidth                                            ' there's no margin for 5x7 font
      temp := 0
    else
      temp := fontTable[base + i]                                ' get column
    repeat j from 0 to fontHeight-1
      if temp & %1                                               ' copy pixels to column
        pixel(x+i,y+j, color,mode)
      else
        pixel(x+i,y+j,!color,mode)
      temp >>= 1

pub chr(ch)
  repeat                                                         ' write 1-4 character(s) to display
    case ch & $FF
      HOME:  cursorX := cursorY := 0                             ' special case: Home
             clear(PAGE)
      NL:    cursorY += fontHeight                               ' special case: Newline
             cursorX := 0
      CR:    cursorX := 0                                        ' special case: Return
      SETX+0..SETX+9:
             cursorX := ((ch & $FF) - SETX) * (fontWidth + 1)    ' special case: set X cursor
      SETY+0..SETY+5:
             cursorY := ((ch & $FF) - SETY) * fontHeight         ' special case: set Y cursor
      other: drawChar(cursorX,cursorY,(ch&$FF),foreColor,drawMode) ' draw character, advance cursor
             cursorX += fontWidth + 1
             if cursorX > (LCDWIDTH - fontWidth)                 ' wraparound
               cursorY += fontHeight
               cursorX := 0
  until (ch >>= 8) == 0

pub hex(value,digits)                                            ' display a hexadecimal number
  value <<= (8 - digits) << 2
  repeat digits
    chr(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))

pub str(address) | ch                                            ' display a string
  repeat while (ch := byte[address++]) <> 0
    chr(ch)

pub dec(value) | divisor, x                                      ' display a decimal number
  x := (value == NEGX)                                           ' check for max negative
  if (value < 0)                                                    
    value := ||(value+x)                                         ' if negative, make positive; adjust for max negative
    chr("-")                                               '  and display sign                                               
  divisor := 1_000_000_000                                       ' initialize divisor                                                   
  repeat 10                                                      ' loop for 10 digits
    if (value => divisor)                                        ' if non-zero digit, display digit, adjust for max negative         
      chr(value / divisor + "0" + x*(divisor == 1))
      value //= divisor                                          ' and digit from value
      result~~                                                   ' flag non-zero found
    elseif (result or (divisor == 1))                                     
      chr("0")                                             ' if zero digit (or only digit) display it
    divisor /= 10                                                ' update divisor


pub rjdec(val,width,pchar) | tmpval, pad

'' Print right-justified decimal value
'' -- val is value to print
'' -- width is width of (padded) field for value
'' -- pchar is [leading] pad character (usually "0" or " ")

'  Original code by Dave Hein
'  Added (with modifications) to FDS by Jon McPhalen

  if (val => 0)                                                  ' if positive
    tmpval := val                                                '  copy value
    pad := width - 1                                             '  make room for 1 digit
  else                                                            
    if (val == NEGX)                                             '  if max negative
      tmpval := POSX                                             '    use max positive for width
    else                                                         '  else
      tmpval := -val                                             '    make positive
    pad := width - 2                                             '  make room for sign and 1 digit                                                                  
  repeat while (tmpval => 10)                                    ' adjust pad for value width > 1
    pad--                                                         
    tmpval /= 10                                                                 
  repeat pad                                                     ' print pad
    chr(pchar)
  dec(val)                                                       ' print value

dat
fontWidth  byte 5                                                ' font width in columns
fontHeight byte 8                                                ' font width in rows
fontStart  byte 0                                                ' first character in font
fontChars  byte 255                                              ' number of characters in font
fontMapWth byte 12, 75                                           ' font map width MSB/LSB
fontTable  byte $00, $00, $00, $00, $00, $3E, $5B, $4F, $5B, $3E ' font with one byte == one col
	byte $3E, $6B, $4F, $6B, $3E, $1C, $3E, $7C, $3E, $1C ' fontWidth bytes for each char
	byte $18, $3C, $7E, $3C, $18, $1C, $57, $7D, $57, $1C
	byte $1C, $5E, $7F, $5E, $1C, $00, $18, $3C, $18, $00
	byte $FF, $E7, $C3, $E7, $FF, $00, $18, $24, $18, $00
	byte $FF, $E7, $DB, $E7, $FF, $30, $48, $3A, $06, $0E
	byte $26, $29, $79, $29, $26, $40, $7F, $05, $05, $07
	byte $40, $7F, $05, $25, $3F, $5A, $3C, $E7, $3C, $5A
	byte $7F, $3E, $1C, $1C, $08, $08, $1C, $1C, $3E, $7F
	byte $14, $22, $7F, $22, $14, $5F, $5F, $00, $5F, $5F
	byte $06, $09, $7F, $01, $7F, $00, $66, $89, $95, $6A
	byte $60, $60, $60, $60, $60, $94, $A2, $FF, $A2, $94
	byte $08, $04, $7E, $04, $08, $10, $20, $7E, $20, $10
	byte $08, $08, $2A, $1C, $08, $08, $1C, $2A, $08, $08
	byte $1E, $10, $10, $10, $10, $0C, $1E, $0C, $1E, $0C
	byte $30, $38, $3E, $38, $30, $06, $0E, $3E, $0E, $06
	byte $00, $00, $00, $00, $00, $00, $00, $5F, $00, $00
	byte $00, $07, $00, $07, $00, $14, $7F, $14, $7F, $14
	byte $24, $2A, $7F, $2A, $12, $23, $13, $08, $64, $62
	byte $36, $49, $56, $20, $50, $00, $08, $07, $03, $00
	byte $00, $1C, $22, $41, $00, $00, $41, $22, $1C, $00
	byte $2A, $1C, $7F, $1C, $2A, $08, $08, $3E, $08, $08
	byte $00, $80, $70, $30, $00, $08, $08, $08, $08, $08
	byte $00, $00, $60, $60, $00, $20, $10, $08, $04, $02
	byte $3E, $51, $49, $45, $3E, $00, $42, $7F, $40, $00
	byte $72, $49, $49, $49, $46, $21, $41, $49, $4D, $33
	byte $18, $14, $12, $7F, $10, $27, $45, $45, $45, $39
	byte $3C, $4A, $49, $49, $31, $41, $21, $11, $09, $07
	byte $36, $49, $49, $49, $36, $46, $49, $49, $29, $1E
	byte $00, $00, $14, $00, $00, $00, $40, $34, $00, $00
	byte $00, $08, $14, $22, $41, $14, $14, $14, $14, $14
	byte $00, $41, $22, $14, $08, $02, $01, $59, $09, $06
	byte $3E, $41, $5D, $59, $4E, $7C, $12, $11, $12, $7C
	byte $7F, $49, $49, $49, $36, $3E, $41, $41, $41, $22
	byte $7F, $41, $41, $41, $3E, $7F, $49, $49, $49, $41
	byte $7F, $09, $09, $09, $01, $3E, $41, $41, $51, $73
	byte $7F, $08, $08, $08, $7F, $00, $41, $7F, $41, $00
	byte $20, $40, $41, $3F, $01, $7F, $08, $14, $22, $41
	byte $7F, $40, $40, $40, $40, $7F, $02, $1C, $02, $7F
	byte $7F, $04, $08, $10, $7F, $3E, $41, $41, $41, $3E
	byte $7F, $09, $09, $09, $06, $3E, $41, $51, $21, $5E
	byte $7F, $09, $19, $29, $46, $26, $49, $49, $49, $32
	byte $03, $01, $7F, $01, $03, $3F, $40, $40, $40, $3F
	byte $1F, $20, $40, $20, $1F, $3F, $40, $38, $40, $3F
	byte $63, $14, $08, $14, $63, $03, $04, $78, $04, $03
	byte $61, $59, $49, $4D, $43, $00, $7F, $41, $41, $41
	byte $02, $04, $08, $10, $20, $00, $41, $41, $41, $7F
	byte $04, $02, $01, $02, $04, $40, $40, $40, $40, $40
	byte $00, $03, $07, $08, $00, $20, $54, $54, $78, $40
	byte $7F, $28, $44, $44, $38, $38, $44, $44, $44, $28
	byte $38, $44, $44, $28, $7F, $38, $54, $54, $54, $18
	byte $00, $08, $7E, $09, $02, $18, $A4, $A4, $9C, $78
	byte $7F, $08, $04, $04, $78, $00, $44, $7D, $40, $00
	byte $20, $40, $40, $3D, $00, $7F, $10, $28, $44, $00
	byte $00, $41, $7F, $40, $00, $7C, $04, $78, $04, $78
	byte $7C, $08, $04, $04, $78, $38, $44, $44, $44, $38
	byte $FC, $18, $24, $24, $18, $18, $24, $24, $18, $FC
	byte $7C, $08, $04, $04, $08, $48, $54, $54, $54, $24
	byte $04, $04, $3F, $44, $24, $3C, $40, $40, $20, $7C
	byte $1C, $20, $40, $20, $1C, $3C, $40, $30, $40, $3C
	byte $44, $28, $10, $28, $44, $4C, $90, $90, $90, $7C
	byte $44, $64, $54, $4C, $44, $00, $08, $36, $41, $00
	byte $00, $00, $77, $00, $00, $00, $41, $36, $08, $00
	byte $02, $01, $02, $04, $02, $3C, $26, $23, $26, $3C
	byte $1E, $A1, $A1, $61, $12, $3A, $40, $40, $20, $7A
	byte $38, $54, $54, $55, $59, $21, $55, $55, $79, $41
	byte $21, $54, $54, $78, $41, $21, $55, $54, $78, $40
	byte $20, $54, $55, $79, $40, $0C, $1E, $52, $72, $12
	byte $39, $55, $55, $55, $59, $39, $54, $54, $54, $59
	byte $39, $55, $54, $54, $58, $00, $00, $45, $7C, $41
	byte $00, $02, $45, $7D, $42, $00, $01, $45, $7C, $40
	byte $F0, $29, $24, $29, $F0, $F0, $28, $25, $28, $F0
	byte $7C, $54, $55, $45, $00, $20, $54, $54, $7C, $54
	byte $7C, $0A, $09, $7F, $49, $32, $49, $49, $49, $32
	byte $32, $48, $48, $48, $32, $32, $4A, $48, $48, $30
	byte $3A, $41, $41, $21, $7A, $3A, $42, $40, $20, $78
	byte $00, $9D, $A0, $A0, $7D, $39, $44, $44, $44, $39
	byte $3D, $40, $40, $40, $3D, $3C, $24, $FF, $24, $24
	byte $48, $7E, $49, $43, $66, $2B, $2F, $FC, $2F, $2B
	byte $FF, $09, $29, $F6, $20, $C0, $88, $7E, $09, $03
	byte $20, $54, $54, $79, $41, $00, $00, $44, $7D, $41
	byte $30, $48, $48, $4A, $32, $38, $40, $40, $22, $7A
	byte $00, $7A, $0A, $0A, $72, $7D, $0D, $19, $31, $7D
	byte $26, $29, $29, $2F, $28, $26, $29, $29, $29, $26
	byte $30, $48, $4D, $40, $20, $38, $08, $08, $08, $08
	byte $08, $08, $08, $08, $38, $2F, $10, $C8, $AC, $BA
	byte $2F, $10, $28, $34, $FA, $00, $00, $7B, $00, $00
	byte $08, $14, $2A, $14, $22, $22, $14, $2A, $14, $08
	byte $AA, $00, $55, $00, $AA, $AA, $55, $AA, $55, $AA
	byte $00, $00, $00, $FF, $00, $10, $10, $10, $FF, $00
	byte $14, $14, $14, $FF, $00, $10, $10, $FF, $00, $FF
	byte $10, $10, $F0, $10, $F0, $14, $14, $14, $FC, $00
	byte $14, $14, $F7, $00, $FF, $00, $00, $FF, $00, $FF
	byte $14, $14, $F4, $04, $FC, $14, $14, $17, $10, $1F
	byte $10, $10, $1F, $10, $1F, $14, $14, $14, $1F, $00
	byte $10, $10, $10, $F0, $00, $00, $00, $00, $1F, $10
	byte $10, $10, $10, $1F, $10, $10, $10, $10, $F0, $10
	byte $00, $00, $00, $FF, $10, $10, $10, $10, $10, $10
	byte $10, $10, $10, $FF, $10, $00, $00, $00, $FF, $14
	byte $00, $00, $FF, $00, $FF, $00, $00, $1F, $10, $17
	byte $00, $00, $FC, $04, $F4, $14, $14, $17, $10, $17
	byte $14, $14, $F4, $04, $F4, $00, $00, $FF, $00, $F7
	byte $14, $14, $14, $14, $14, $14, $14, $F7, $00, $F7
	byte $14, $14, $14, $17, $14, $10, $10, $1F, $10, $1F
	byte $14, $14, $14, $F4, $14, $10, $10, $F0, $10, $F0
	byte $00, $00, $1F, $10, $1F, $00, $00, $00, $1F, $14
	byte $00, $00, $00, $FC, $14, $00, $00, $F0, $10, $F0
	byte $10, $10, $FF, $10, $FF, $14, $14, $14, $FF, $14
	byte $10, $10, $10, $1F, $00, $00, $00, $00, $F0, $10
	byte $FF, $FF, $FF, $FF, $FF, $F0, $F0, $F0, $F0, $F0
	byte $FF, $FF, $FF, $00, $00, $00, $00, $00, $FF, $FF
	byte $0F, $0F, $0F, $0F, $0F, $38, $44, $44, $38, $44
	byte $7C, $2A, $2A, $3E, $14, $7E, $02, $02, $06, $06
	byte $02, $7E, $02, $7E, $02, $63, $55, $49, $41, $63
	byte $38, $44, $44, $3C, $04, $40, $7E, $20, $1E, $20
	byte $06, $02, $7E, $02, $02, $99, $A5, $E7, $A5, $99
	byte $1C, $2A, $49, $2A, $1C, $4C, $72, $01, $72, $4C
	byte $30, $4A, $4D, $4D, $30, $30, $48, $78, $48, $30
	byte $BC, $62, $5A, $46, $3D, $3E, $49, $49, $49, $00
	byte $7E, $01, $01, $01, $7E, $2A, $2A, $2A, $2A, $2A
	byte $44, $44, $5F, $44, $44, $40, $51, $4A, $44, $40
	byte $40, $44, $4A, $51, $40, $00, $00, $FF, $01, $03
	byte $E0, $80, $FF, $00, $00, $08, $08, $6B, $6B, $08
	byte $36, $12, $36, $24, $36, $06, $0F, $09, $0F, $06
	byte $00, $00, $18, $18, $00, $00, $00, $10, $10, $00
	byte $30, $40, $FF, $01, $01, $00, $1F, $01, $01, $1E
	byte $00, $19, $1D, $17, $12, $00, $3C, $3C, $3C, $3C
	byte $00, $00, $00, $00, $00
