con
   SSD1675_DRIVER_CONTROL    = $01
   SSD1675_GATE_VOLTAGE      = $03
   SSD1675_SOURCE_VOLTAGE    = $04
   SSD1675_DEEP_SLEEP        = $10
   SSD1675_DATA_MODE         = $11
   SSD1675_SW_RESET          = $12
   SSD1675_HV_READY          = $14
   SSD1675_VCI_READY         = $15
   SSD1675_TEMP_WRITE        = $1A
   SSD1675_MASTER_ACTIVATE   = $20
   SSD1675_DISP_CTRL1        = $21
   SSD1675_DISP_CTRL2        = $22
   SSD1675_WRITE_RAM1        = $24
   SSD1675_WRITE_RAM2        = $26
   SSD1675_WRITE_VCOM        = $2C
   SSD1675_READ_OTP          = $2D
   SSD1675_READ_STATUS       = $2F
   SSD1675_WRITE_LUT         = $32
   SSD1675_WRITE_DUMMY       = $3A
   SSD1675_WRITE_GATELINE    = $3B
   SSD1675_WRITE_BORDER      = $3C
   SSD1675_SET_RAMXPOS       = $44
   SSD1675_SET_RAMYPOS       = $45
   SSD1675_SET_RAMXCOUNT     = $4E
   SSD1675_SET_RAMYCOUNT     = $4F
   SSD1675_SET_ANALOGBLOCK   = $74
   SSD1675_SET_DIGITALBLOCK  = $7E

   BUSY_WAIT_TIME            = 500

   SRAM_READ                 = $03     ' SRAM read command
   SRAM_WRITE                = $02     ' SRAM write command
   SRAM_RDSR                 = $05     ' SRAM read status register command
   SRAM_WRSR                 = $01     ' SRAM write status register command
   SRAM_SeqMd                = 1 << 6  ' SRAM status set sequential mode

   HOME                      = $01     ' Clears display and does setCursor(0,0)
   CR                        = $0D     ' Sets X cursor to 0
   NL                        = $0A     ' Sets Y cursor to Y+1
   SETX                      = $80     ' Sets X character cursor to ???? - SETX
   SETY                      = $B0     ' Sets Y character cursor to ???? - SETY

dat                                    ' Initialization data
LUT_DATA byte $80,$60,$40,$00,$00,$00,$00 '' LUT0: BB:     VS 0 ~7
         byte $10,$60,$20,$00,$00,$00,$00 '' LUT1: BW:     VS 0 ~7
         byte $80,$60,$40,$00,$00,$00,$00 '' LUT2: WB:     VS 0 ~7
         byte $10,$60,$20,$00,$00,$00,$00 '' LUT3: WW:     VS 0 ~7
         byte $00,$00,$00,$00,$00,$00,$00 '' LUT4: VCOM:   VS 0 ~7

         byte $03,$03,$00,$00,$02         '' TP0 A~D RP0
         byte $09,$09,$00,$00,$02         '' TP1 A~D RP1
         byte $03,$03,$00,$00,$02         '' TP2 A~D RP2
         byte $00,$00,$00,$00,$00         '' TP3 A~D RP3
         byte $00,$00,$00,$00,$00         '' TP4 A~D RP4
         byte $00,$00,$00,$00,$00         '' TP5 A~D RP5
         byte $00,$00,$00,$00,$00         '' TP6 A~D RP6

         byte $15,$41,$A8,$32,$30,$0A

bitTable byte $01,$02,$04,$08,$10,$20,$40,$80

sample   byte "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
         byte "Curabitur adipiscing ante sed nibh tincidunt feugiat. "
         byte "Maecenas enim massa, fringilla sed malesuada et, malesuada "
         byte "sit amet turpis. Sed porttitor neque ut ante pretium vitae "
         byte "malesuada nunc bibendum. Nullam aliquet ultrices massa eu "
         byte "hendrerit. Ut sed nisi lorem. In vestibulum purus a tortor "
         byte "imperdiet posuere. ",0

var
  long _width, _height, buffer_size, buffer_addr, _busy_pin, _ena_pin
  long _mosi_pin, _sclk_pin, _dc_pin, _reset_pin, _ecs_pin, _srcs_pin, _miso_pin
  byte buffer[(256 * 128) / 8]         ' Note: size rounded up to nearest byte
  byte cursorX,cursorY                 ' Pixel cursor location

pub resetAll                           ' Reset everything
   outa[_ena_pin]~~
   dira[_ena_pin]~~                    ' Turn off power first
   outa[_ena_pin]~
   waitcnt(clkfreq/10 + cnt)
   dira[_ena_pin]~                     ' Pullup on ENA enables power

pub SSD1675( width, height, MOSI, SCLK, DC, RESET, ECS, SRCS, MISO, BUSY, ENA)
                                       ' Save I/O pin numbers and screen size
  _mosi_pin  := MOSI
  _sclk_pin  := SCLK
  _dc_pin    := DC                     ' save I/O pin numbers
  _reset_pin := RESET
  _ecs_pin   := ECS
  _srcs_pin  := SRCS
  _miso_pin  := MISO
  _busy_pin  := BUSY
  _ena_pin   := ENA
  
  _width     := width
  _height    := height
  
  if (_height // 8) <> 0                ' round up height to multiple of 8
    _height += 8 - (_height // 8)
  buffer_size := width * height / 8
  buffer_addr := @buffer

PUB sampleCode | i
   begin(false)
   clearBuffer
   setCursor(0, 0)
   str(@sample)
   display
   waitcnt(5*clkfreq + cnt)
   clearBuffer
   repeat i from 0 to _width step 4
      drawLine(0, 0, i, _height-1)
   repeat i from 0 to _height step 4
      drawLine(_width-1,0,0,i)
   display

pub busy_wait                          ' Wait for busy signal to end
  if _busy_pin >= 0
    repeat until ina[_busy_pin]        ' wait for busy low
      waitcnt(clkfreq/100 + cnt)
  else                                 ' Busy pin not available,
    waitcnt(BUSY_WAIT_TIME*(clkfreq/1000) + cnt) ' use default wait

pub hardwareReset                      ' Do a hardware reset
  outa[_reset_pin]~~
  dira[_reset_pin]~~                   ' start with rst high
  waitcnt(clkfreq/100 + cnt)           '  for 10ms
  outa[_reset_pin]~
  waitcnt(clkfreq/100 + cnt)           ' take rst low for 10ms
  outa[_reset_pin]~~                   '  then leave it high

pub update                             ' Signal the display to update
  doCommand(SSD1675_DISP_CTRL2,$C7,1)
  doCommand(SSD1675_MASTER_ACTIVATE,0,0)
  busy_wait
  if _busy_pin < 0                     ' If no BUSY pin, use default delay
    waitcnt(clkfreq + cnt)

pub powerUp(reset)                     ' Start up the display
  if reset
     hardwareReset                     ' First a hardware reset
     waitcnt(clkfreq/10 + cnt)         '  with a 100ms delay
     busy_wait
  doCommand(SSD1675_SW_RESET,0,0)                               ' software reset
  busy_wait                                                     '  wait for not busy
  doCommand(SSD1675_SET_ANALOGBLOCK,$54,1)                      ' set analog block control
  doCommand(SSD1675_SET_DIGITALBLOCK,$3B,1)                     ' set digital block control
  doCommand(SSD1675_DRIVER_CONTROL,$FA|$01<<8|$00<<16,3)        ' driver output control
  doCommand(SSD1675_DATA_MODE,$03,1)                            ' data entry sequence
  doCommand(SSD1675_SET_RAMXPOS,$00|$0F<<8,2)                   ' set ram X start/end postion
  doCommand(SSD1675_SET_RAMYPOS,$00|$00<<8|$F9<<16|$00<<24,4)   ' set ram Y start/end postion
  doCommand(SSD1675_WRITE_BORDER,$03,1)                         ' border color
  doCommand(SSD1675_WRITE_VCOM,$70,1)                           ' Vcom Voltage
  doCommand(SSD1675_GATE_VOLTAGE,LUT_DATA[70],1)                ' set gate voltage
  doCommand(SSD1675_SOURCE_VOLTAGE,LUT_DATA[71]|LUT_DATA[72]<<8|LUT_DATA[73]<<16,3) ' set source voltage
  doCommand(SSD1675_WRITE_DUMMY,LUT_DATA[74],1)                 ' set dummy line period
  doCommand(SSD1675_WRITE_GATELINE,LUT_DATA[75],1)              ' set gate line width
  doCommandPart1(SSD1675_WRITE_LUT)                             ' copy table to LUT
  doCommandPart2(@LUT_DATA,70)
  doCommand(SSD1675_SET_RAMXCOUNT,0,1)                          ' set RAM x address count to 0;
  doCommand(SSD1675_SET_RAMYCOUNT,$F9|$00<<8,2)                 ' set RAM y address count to $127;
  busy_wait

pub powerDown                          ' Power down the display
  '' Only deep sleep if we can get out of it
  if _reset_pin >= 0                   ' Don't use deep sleep if no RESET
    doCommand(SSD1675_DEEP_SLEEP,$01,1)
    waitcnt(clkfreq/10 + cnt)
  else
    doCommand(SSD1675_SW_RESET,0,0)
    busy_wait

pub setRAMAddress(x,y)                 ' Set RAM address pointer
  doCommand(SSD1675_SET_RAMXCOUNT,$00,1)
  doCommand(SSD1675_SET_RAMYCOUNT,$F9|$00<<8,2) ' set RAM Y address counter

pub doCommandPart1(cmd)                ' Output command byte
  outa[_ecs_pin]~~
  dira[_ecs_pin]~~                     ' Chip select idles high
  outa[_dc_pin]~                       ' D/C low for command bytes
  dira[_dc_pin]~~
  outa[_ecs_pin]~                      ' Chip select low
  shiftOut(_mosi_pin,_sclk_pin,cmd,8)  ' shift out one byte

pub doCommandPart2(argPtr,count)       ' Output data byte(s)
  outa[_dc_pin]~~                      ' Change D/C to high for data bytes
  repeat count
    shiftOut(_mosi_pin,_sclk_pin,byte[argPtr++],8) ' Shift out any bytes of data
  outa[_ecs_pin]~~                     ' Deselect the chip
  dira[_dc_pin]~
  dira[_ecs_pin]~                      ' Let control lines float. MOSI
  dira[_mosi_pin]~                     '  as well. (MISO already an input)

pub doCommand(cmd,args,count)          ' output 0-4 bytes of immediate data
  doCommandPart1(cmd)
  doCommandPart2(@args,count)

con
   #0, EPD_BLACK,EPD_WHITE,EPD_INVERSE,EPD_RED,EPD_GRAY,EPD_DARK,EPD_LIGHT

pri shiftOut(MOSI,SCK,bits,data)      ' Shift out data to MOSI
   repeat bits
      outa[MOSI] := data >> (bits-1)  ' Order is MSB -> LSB
      outa[SCK]~~
      outa[SCK]~                      ' Clock out data
      data <<= 1

pri shiftIn(MISO,SCK,bits) : data      ' Shift in data from MISO
   repeat bits
      data <<= 1                       ' Order is MSB -> LSB
      data |= ina[MISO]
      outa[SCK]~~
      outa[SCK]~                       ' Clock in data

pri shiftInOut(MOSI,SCK,MISO,bits,out):inp ' Shift out data to MOSI. At the
   repeat bits                         '  same time, shift in data from MISO
      inp <<= 1
      outa[MOSI] := out >> (bits-1)    ' Order is MSB -> LSB
      inp |= ina[MISO]
      outa[SCK]~~
      outa[SCK]~                       ' Clock out data
      out <<= 1

pub drawPixel(x,y,color)
   if (_height & 7) <> 0               ' Round up height to multiple of 8
      _height += 8 - (_height & 7)
   if (x < 0) or (x >= _width) or (y < 0) or (y >= _height)
      return
   if color == 0
      buffer[x + (y>>3)<<8] &= !bitTable[y&7] ' Set the bit to 0
   else
      buffer[x + (y>>3)<<8] |= bitTable[y&7] ' Set the bit to 1
   
pub display | i, t                     ' Transfer data from buffer to display
   powerUp(false)
   setRAMAddress(0, 0)                 ' Set X & Y ram counters
   outa[_dc_pin]~~                     ' D/C high for data
   dira[_dc_pin]~~
   outa[_srcs_pin]~~
   dira[_srcs_pin]~~                   ' Prepare to select SRAM
   outa[_srcs_pin]~
   shiftOut(_mosi_pin,_sclk_pin,8,SRAM_READ) ' Read SRAM
   shiftOut(_mosi_pin,_sclk_pin,16,buffer_addr)
   '' first data byte from SRAM will be transfered in at the same time
   '' as the EPD command is transferred out
   t := shiftInOut(_mosi_pin,_sclk_pin,_miso_pin,8,0)
   outa[_dc_pin]~
   repeat i from 0 to buffer_size-1
      t := shiftInOut(_mosi_pin,_sclk_pin,_miso_pin,8,t)
   dira[_ecs_pin]~
   dira[_srcs_pin]~
   dira[_dc_pin]~
   update
   powerDown

pub drawLine(x0,y0,x1,y1) | steep, t, dx, dy, err, ystep
   if steep := ||(y1 - y0) > ||(x1 - x0)
      t := x0
      x0 := y0
      y0 := t
      t := x1
      x1 := y1
      y1 := t
   if x0 > x1
      t := x0
      x0 := x1
      x1 := t
      t := y0
      y0 := y1
      y1 := t
   dx := x1 - x0
   dy := ||(y1 - y0)
   err := dx / 2
   if y0 < y1
      ystep := 1
   else
      ystep := -1
   repeat while x0 =< x1
      if steep
         drawPixel(y0,x0,1)
      else
         drawPixel(x0,y0,1)
      err -= dy
      if err < 0
         y0 += ystep
         err += dx
      x0++

pub begin(doReset)
   if false                            ' Not using SRAM
      outa[_srcs_pin]~~
      dira[_srcs_pin]~~
      outa[_srcs_pin]~
      shiftOut(_mosi_pin,_sclk_pin,8,SRAM_WRSR)
      shiftOut(_mosi_pin,_sclk_pin,8,SRAM_SeqMd)
      outa[_srcs_pin]~~
      dira[_srcs_pin]~
   outa[_dc_pin]~
   dira[_dc_pin]~~
   outa[_ecs_pin]~~
   dira[_ecs_pin]~~
   outa[_sclk_pin]~
   dira[_sclk_pin]~~
   dira[_busy_pin]~
   if doReset
      hardwareReset

pub setCursor(x,y)
  cursorX := x
  cursorY := y

pub drawChar(x,y,ch) | temp, i, j, base                          ' draw a character on the display
  base := (ch - fontStart) * fontWidth                           ' base of font for character
  repeat i from 0 to fontWidth
    if i == fontWidth                                            ' there's no margin for 5x7 font
      temp := 0
    else
      temp := fontTable[base + i]                                ' get column
    repeat j from 0 to fontHeight-1
      if temp & %1                                               ' copy pixels to column
        drawPixel(x+i,y+j,1)
      else
        drawPixel(x+i,y+j,0)
      temp >>= 1

pub chr(ch)
  repeat                                                         ' write 1-4 character(s) to display
    case ch & $FF
      HOME:  cursorX := cursorY := 0                             ' special case: Home
             clearBuffer
      NL:    cursorY += fontHeight                               ' special case: Newline
             cursorX := 0
      CR:    cursorX := 0                                        ' special case: Return
      SETX+0..SETX+39:
             cursorX := ((ch & $FF) - SETX) * (fontWidth + 1)    ' special case: set X cursor
      SETY+0..SETY+14:
             cursorY := ((ch & $FF) - SETY) * fontHeight         ' special case: set Y cursor
      other: drawChar(cursorX,cursorY,(ch&$FF))                  ' draw character, advance cursor
             cursorX += fontWidth + 1
             if cursorX > (_width - fontWidth)                   ' wraparound
               cursorY += fontHeight
               cursorX := 0
  until (ch >>= 8) == 0

pub str(s)
   repeat strsize(s)
      chr(byte[s++])

pub clearBuffer
   bytefill(@buffer,$00,buffer_size)

pub clearDisplay
   clearBuffer
   display
   waitcnt(clkfreq/10 + cnt)
   display

dat
fontWidth  byte 5                                                ' font width in columns
fontHeight byte 8                                                ' font width in rows
fontStart  byte 0                                                ' first character in font
fontChars  byte 255                                              ' number of characters in font
fontMapWth byte 12, 75                                           ' font map width MSB/LSB
fontTable  byte $00, $00, $00, $00, $00, $3E, $5B, $4F, $5B, $3E ' font with one byte == one col
           byte $3E, $6B, $4F, $6B, $3E, $1C, $3E, $7C, $3E, $1C ' fontWidth bytes for each char
           byte $18, $3C, $7E, $3C, $18, $1C, $57, $7D, $57, $1C
           byte $1C, $5E, $7F, $5E, $1C, $00, $18, $3C, $18, $00
           byte $FF, $E7, $C3, $E7, $FF, $00, $18, $24, $18, $00
           byte $FF, $E7, $DB, $E7, $FF, $30, $48, $3A, $06, $0E
           byte $26, $29, $79, $29, $26, $40, $7F, $05, $05, $07
           byte $40, $7F, $05, $25, $3F, $5A, $3C, $E7, $3C, $5A
           byte $7F, $3E, $1C, $1C, $08, $08, $1C, $1C, $3E, $7F
           byte $14, $22, $7F, $22, $14, $5F, $5F, $00, $5F, $5F
           byte $06, $09, $7F, $01, $7F, $00, $66, $89, $95, $6A
           byte $60, $60, $60, $60, $60, $94, $A2, $FF, $A2, $94
           byte $08, $04, $7E, $04, $08, $10, $20, $7E, $20, $10
           byte $08, $08, $2A, $1C, $08, $08, $1C, $2A, $08, $08
           byte $1E, $10, $10, $10, $10, $0C, $1E, $0C, $1E, $0C
           byte $30, $38, $3E, $38, $30, $06, $0E, $3E, $0E, $06
           byte $00, $00, $00, $00, $00, $00, $00, $5F, $00, $00
           byte $00, $07, $00, $07, $00, $14, $7F, $14, $7F, $14
           byte $24, $2A, $7F, $2A, $12, $23, $13, $08, $64, $62
           byte $36, $49, $56, $20, $50, $00, $08, $07, $03, $00
           byte $00, $1C, $22, $41, $00, $00, $41, $22, $1C, $00
           byte $2A, $1C, $7F, $1C, $2A, $08, $08, $3E, $08, $08
           byte $00, $80, $70, $30, $00, $08, $08, $08, $08, $08
           byte $00, $00, $60, $60, $00, $20, $10, $08, $04, $02
           byte $3E, $51, $49, $45, $3E, $00, $42, $7F, $40, $00
           byte $72, $49, $49, $49, $46, $21, $41, $49, $4D, $33
           byte $18, $14, $12, $7F, $10, $27, $45, $45, $45, $39
           byte $3C, $4A, $49, $49, $31, $41, $21, $11, $09, $07
           byte $36, $49, $49, $49, $36, $46, $49, $49, $29, $1E
           byte $00, $00, $14, $00, $00, $00, $40, $34, $00, $00
           byte $00, $08, $14, $22, $41, $14, $14, $14, $14, $14
           byte $00, $41, $22, $14, $08, $02, $01, $59, $09, $06
           byte $3E, $41, $5D, $59, $4E, $7C, $12, $11, $12, $7C
           byte $7F, $49, $49, $49, $36, $3E, $41, $41, $41, $22
           byte $7F, $41, $41, $41, $3E, $7F, $49, $49, $49, $41
           byte $7F, $09, $09, $09, $01, $3E, $41, $41, $51, $73
           byte $7F, $08, $08, $08, $7F, $00, $41, $7F, $41, $00
           byte $20, $40, $41, $3F, $01, $7F, $08, $14, $22, $41
           byte $7F, $40, $40, $40, $40, $7F, $02, $1C, $02, $7F
           byte $7F, $04, $08, $10, $7F, $3E, $41, $41, $41, $3E
           byte $7F, $09, $09, $09, $06, $3E, $41, $51, $21, $5E
           byte $7F, $09, $19, $29, $46, $26, $49, $49, $49, $32
           byte $03, $01, $7F, $01, $03, $3F, $40, $40, $40, $3F
           byte $1F, $20, $40, $20, $1F, $3F, $40, $38, $40, $3F
           byte $63, $14, $08, $14, $63, $03, $04, $78, $04, $03
           byte $61, $59, $49, $4D, $43, $00, $7F, $41, $41, $41
           byte $02, $04, $08, $10, $20, $00, $41, $41, $41, $7F
           byte $04, $02, $01, $02, $04, $40, $40, $40, $40, $40
           byte $00, $03, $07, $08, $00, $20, $54, $54, $78, $40
           byte $7F, $28, $44, $44, $38, $38, $44, $44, $44, $28
           byte $38, $44, $44, $28, $7F, $38, $54, $54, $54, $18
           byte $00, $08, $7E, $09, $02, $18, $A4, $A4, $9C, $78
           byte $7F, $08, $04, $04, $78, $00, $44, $7D, $40, $00
           byte $20, $40, $40, $3D, $00, $7F, $10, $28, $44, $00
           byte $00, $41, $7F, $40, $00, $7C, $04, $78, $04, $78
           byte $7C, $08, $04, $04, $78, $38, $44, $44, $44, $38
           byte $FC, $18, $24, $24, $18, $18, $24, $24, $18, $FC
           byte $7C, $08, $04, $04, $08, $48, $54, $54, $54, $24
           byte $04, $04, $3F, $44, $24, $3C, $40, $40, $20, $7C
           byte $1C, $20, $40, $20, $1C, $3C, $40, $30, $40, $3C
           byte $44, $28, $10, $28, $44, $4C, $90, $90, $90, $7C
           byte $44, $64, $54, $4C, $44, $00, $08, $36, $41, $00
           byte $00, $00, $77, $00, $00, $00, $41, $36, $08, $00
           byte $02, $01, $02, $04, $02, $3C, $26, $23, $26, $3C
           byte $1E, $A1, $A1, $61, $12, $3A, $40, $40, $20, $7A
           byte $38, $54, $54, $55, $59, $21, $55, $55, $79, $41
           byte $21, $54, $54, $78, $41, $21, $55, $54, $78, $40
           byte $20, $54, $55, $79, $40, $0C, $1E, $52, $72, $12
           byte $39, $55, $55, $55, $59, $39, $54, $54, $54, $59
           byte $39, $55, $54, $54, $58, $00, $00, $45, $7C, $41
           byte $00, $02, $45, $7D, $42, $00, $01, $45, $7C, $40
           byte $F0, $29, $24, $29, $F0, $F0, $28, $25, $28, $F0
           byte $7C, $54, $55, $45, $00, $20, $54, $54, $7C, $54
           byte $7C, $0A, $09, $7F, $49, $32, $49, $49, $49, $32
           byte $32, $48, $48, $48, $32, $32, $4A, $48, $48, $30
           byte $3A, $41, $41, $21, $7A, $3A, $42, $40, $20, $78
           byte $00, $9D, $A0, $A0, $7D, $39, $44, $44, $44, $39
           byte $3D, $40, $40, $40, $3D, $3C, $24, $FF, $24, $24
           byte $48, $7E, $49, $43, $66, $2B, $2F, $FC, $2F, $2B
           byte $FF, $09, $29, $F6, $20, $C0, $88, $7E, $09, $03
           byte $20, $54, $54, $79, $41, $00, $00, $44, $7D, $41
           byte $30, $48, $48, $4A, $32, $38, $40, $40, $22, $7A
           byte $00, $7A, $0A, $0A, $72, $7D, $0D, $19, $31, $7D
           byte $26, $29, $29, $2F, $28, $26, $29, $29, $29, $26
           byte $30, $48, $4D, $40, $20, $38, $08, $08, $08, $08
           byte $08, $08, $08, $08, $38, $2F, $10, $C8, $AC, $BA
           byte $2F, $10, $28, $34, $FA, $00, $00, $7B, $00, $00
           byte $08, $14, $2A, $14, $22, $22, $14, $2A, $14, $08
           byte $AA, $00, $55, $00, $AA, $AA, $55, $AA, $55, $AA
           byte $00, $00, $00, $FF, $00, $10, $10, $10, $FF, $00
           byte $14, $14, $14, $FF, $00, $10, $10, $FF, $00, $FF
           byte $10, $10, $F0, $10, $F0, $14, $14, $14, $FC, $00
           byte $14, $14, $F7, $00, $FF, $00, $00, $FF, $00, $FF
           byte $14, $14, $F4, $04, $FC, $14, $14, $17, $10, $1F
           byte $10, $10, $1F, $10, $1F, $14, $14, $14, $1F, $00
           byte $10, $10, $10, $F0, $00, $00, $00, $00, $1F, $10
           byte $10, $10, $10, $1F, $10, $10, $10, $10, $F0, $10
           byte $00, $00, $00, $FF, $10, $10, $10, $10, $10, $10
           byte $10, $10, $10, $FF, $10, $00, $00, $00, $FF, $14
           byte $00, $00, $FF, $00, $FF, $00, $00, $1F, $10, $17
           byte $00, $00, $FC, $04, $F4, $14, $14, $17, $10, $17
           byte $14, $14, $F4, $04, $F4, $00, $00, $FF, $00, $F7
           byte $14, $14, $14, $14, $14, $14, $14, $F7, $00, $F7
           byte $14, $14, $14, $17, $14, $10, $10, $1F, $10, $1F
           byte $14, $14, $14, $F4, $14, $10, $10, $F0, $10, $F0
           byte $00, $00, $1F, $10, $1F, $00, $00, $00, $1F, $14
           byte $00, $00, $00, $FC, $14, $00, $00, $F0, $10, $F0
           byte $10, $10, $FF, $10, $FF, $14, $14, $14, $FF, $14
           byte $10, $10, $10, $1F, $00, $00, $00, $00, $F0, $10
           byte $FF, $FF, $FF, $FF, $FF, $F0, $F0, $F0, $F0, $F0
           byte $FF, $FF, $FF, $00, $00, $00, $00, $00, $FF, $FF
           byte $0F, $0F, $0F, $0F, $0F, $38, $44, $44, $38, $44
           byte $7C, $2A, $2A, $3E, $14, $7E, $02, $02, $06, $06
           byte $02, $7E, $02, $7E, $02, $63, $55, $49, $41, $63
           byte $38, $44, $44, $3C, $04, $40, $7E, $20, $1E, $20
           byte $06, $02, $7E, $02, $02, $99, $A5, $E7, $A5, $99
           byte $1C, $2A, $49, $2A, $1C, $4C, $72, $01, $72, $4C
           byte $30, $4A, $4D, $4D, $30, $30, $48, $78, $48, $30
           byte $BC, $62, $5A, $46, $3D, $3E, $49, $49, $49, $00
           byte $7E, $01, $01, $01, $7E, $2A, $2A, $2A, $2A, $2A
           byte $44, $44, $5F, $44, $44, $40, $51, $4A, $44, $40
           byte $40, $44, $4A, $51, $40, $00, $00, $FF, $01, $03
           byte $E0, $80, $FF, $00, $00, $08, $08, $6B, $6B, $08
           byte $36, $12, $36, $24, $36, $06, $0F, $09, $0F, $06
           byte $00, $00, $18, $18, $00, $00, $00, $10, $10, $00
           byte $30, $40, $FF, $01, $01, $00, $1F, $01, $01, $1E
           byte $00, $19, $1D, $17, $12, $00, $3C, $3C, $3C, $3C
           byte $00, $00, $00, $00, $00
