{{ Uses a SSD1675 controller for the display}}
con
   _clkfreq = 80_000_000
   _clkmode = xtal1 + pll16x

   version  = 10
   
   ENApin   = -1                       ' Power enable
   BUSYpin  = -1                       ' Controller busy
   RESETpin = 0                        ' Reset
   SDCSpin  = 1                        ' SD card CS
   SRCSpin  = 2                        ' SRAM CS
   DCpin    = 3                        ' Data / Command
   ECSpin   = 4                        ' eInk CS
   MOSIpin  = 5                        ' Prop out, Display in
   MISOpin  = 6                        ' Prop in, Display out
   SCKpin   = 7                        ' Clock
   Servo    = 16                       ' Servo/Sensor pins
   QwiicC   = 17                       ' QWIIC Clock
   QwiicD   = 18                       ' QWIIC Data
   TxPin    = 30                       ' Transmit data to WiFi
   RxPin    = 31                       ' Receive data from WiFi
   
   Baud     = 115200

   WiFi     = 0                        ' Serial port for WiFi module
   OLED     = 4                        ' I2C port for OLED96 module
   
   BS       = $08                      ' Backspace
   CR       = $0D                      ' Carriage return
   LF       = $0A                      ' Line feed
   DEL      = $7F                      ' Delete
   NL       = CR | LF<<8               ' Newline

   eWidth   = 250                      ' Display width in pixels
   eHeight  = 122                      ' Display height in pixels
                                       ' 
   lineLen  = 128                      ' Output buffer size
   quote    = 34                       ' Quote character
   nullPtr  = 0                        ' Null pointer
   caseBit  = "a"-"A"                  ' For uppercase <-> lowercase
    
   WrStat   = $01_00                   ' Write Status / Data
   WrData   = $02_000000               ' Write Data / Address / Data
   RdData   = $03_000000               ' Read Data / Address / (Data)
   Status   = $05                      ' Read Status / (Data)
   WrtEna   = $06                      ' Write Enable

   ModeByte = %00_00000_1              ' SRAM Byte mode
   ModePage = %10_00000_1              ' SRAM Page mode
   ModeSeq  = %01_00000_1              ' SRAM Sequential mode

   maxData  = 31                       ' Highest SRAM location to be tested
var
   long outputPtr, outBuffPtr, NLout, BSout, rand
   byte filename[16], outBuff[lineLen+1]
   byte buffer[maxData]                ' Random byte values

dat
hex  byte  "0123456789ABCDEF"

obj
   fsrw:  "fsrw"                       ' SD card FAT file system driver
   eInk:  "SSD1675_eInk_Driver"        ' Adafruit e-Ink driver (for SSD1675)
   ser:   "FullDuplexSerial4pnf"       ' 4-port serial driver
   dsp:   "QWIIC_OLED_Driver"          ' QWIIC I2C OLED driver

PUB main | err, i, r
   BSout := BS
   NLout := CR | LF<<8
   eInk.resetAll                       ' Reset SD, SRAM, eInk
   rand := $BEEFBEEF
   repeat i from 0 to maxData
      buffer[i] := (?rand) & $FF
   dsp.init(QwiicC,dsp#QWIIC_OLED)
   dsp.flipVertical(true)
   dsp.flipHorizontal(true)
   ser.Init                            ' Initialize console serial port
   ser.AddPort(WiFi,RxPin,TxPin,%0000,Baud)
   ser.Start
   dsp.str(string(dsp#SETX,dsp#SETY,"TstEInk "))
   dsp.chr(((version/10)+"0") | ((version//10)+"0")<<8 | CR<<16 | LF<<24)
   dsp.updateDisplay
   sendRecv(WrtEna,0,0)                ' Enable writes to status register
   sendRecv(WrStat|ModeSeq,0,0)        ' Switch to sequential mode
   dsp.str(string(dsp#SETX,dsp#SETY+1,"SRAM #"))
   dsp.dec(testSRAM1)
   dsp.updateDisplay
   eInk.SSD1675(eWidth,eHeight,MOSIpin,SCKpin,DCpin,RESETpin,ECSpin, {
              } SRCSpin,MISOpin,BUSYpin,ENApin)
   eInk.powerUp(false)                 ' Try first without reset
   if (err := \listDirectory) < -1
      dsp.str(string(dsp#SETX+0,dsp#SETY+1,"Err: "))
      dsp.dec(err)
      dsp.updateDisplay
      repeat
         waitcnt(clkfreq + cnt)
   eInk.powerDown
   repeat
      waitcnt(clkfreq + cnt)

PRI testSRAM1 : result | i, j
   repeat i from 0 to maxData          ' Fill SRAM with random bytes
      sendRecv(WrtEna,0,0)
      sendRecv(WrData|i,-1,@buffer+i)
   repeat i from 0 to maxData          ' See if they match
      if sendRecv(RdData|i,1,0) <> buffer[i]
         result++
   ifnot result
      repeat i from maxData to 0       ' Fill SRAM with random bytes
         sendRecv(WrtEna,0,0)          '  this time in reverse order
         sendRecv(WrData|i,-1,@buffer+i)
      repeat i from maxData to 0       ' See if they match
         if sendRecv(RdData|i,1,0) <> buffer[i]
            result++

PUB listDirectory : result | count, p, c
   count := 3
   repeat until fsrw.mount_explicit(MISOpin,SCKpin,MOSIpin,SDCSpin) == 0
      ifnot count--
         outputFormat(string("Can't mount SD card\n"))
         outputFlush(OLED)
         return 0
   fsrw.opendir
   outputFormat(string("Root Directory\n"))
   outputFlush(OLED)
   repeat while fsrw.nextfile(@filename) == 0
      outputFormat(string("File: %s\n"))
      outputItem(@filename)
      outputFlush(OLED)
   fsrw.unmount
   return 0

PRI pinh(pin)
   outa[pin]~~
   dira[pin]~~

PRI pinl(pin)
   outa[pin]~
   dira[pin]~~

PRI pinr(pin) : result
   result := ina[pin]

PRI pinw(pin,v)
   outa[pin] := v
   dira[pin]~~

PRI pinf(pin)
   dira[pin]~

PRI waitus(x)
   waitcnt(x*(clkfreq/1_000_000) + cnt)

PUB sendRecv(sD,rCt,rA):result | i,m   ' Send and possibly receive
'  sD  = Value to be sent MSB first, right justified in parameter
'  rCt = Number of bytes to be transferred (>0 received, <0 sent)
'  rA  = 0 or address of buffer area.  If 0, @result is used
   if rA == 0                          ' Use result if rA is zero
      rA := @result
   pinh(SRCSpin)                       ' Disable SRAM if present
   pinl(SCKpin)
   pinl(MOSIpin)                       ' Set up the various I/O pins
   waitus(2)
   pinl(SRCSpin)
   waitus(2)
   m := $FF000000
   repeat until m == 0 or sD & m <> 0  ' Set up sending mask
      m >>= 8
   m &= $80808080                      ' Start with most significant bit
   repeat until m == 0
      pinw(MOSIpin,((sD & m) <> 0) & 1) ' Output bit to be sent
      waitus(2)
      pinh(SCKpin)                     ' Toggle clock pulse
      waitus(2)
      pinl(SCKpin)
      waitus(2)
      m >>= 1                          ' Advance mask towards LSB
   if rCt < 0
      repeat i from 0 to -rCt-1
         m := $80                      ' Set up sending mask
         sD := byte[rA+i]              '  and get data byte
         repeat until m == 0
            pinw(MOSIpin,((sD & m) <> 0) & 1)
            waitus(2)
            pinh(SCKpin)               ' Output bit to be sent
            waitus(2)
            pinl(SCKpin)               ' Toggle clock pulse
            waitus(2)
            m >>= 1                    ' Advance mask towards LSB
   pinf(MOSIpin)                        ' Don't need DIO pin now
   if rCt > 0
      repeat i from 0 to rCt-1
         m := $80                      ' Set up receiving mask
         sD~                           ' Make sure byte is zeroed
         repeat until m == 0
            waitus(2)
            if pinr(MISOpin) <> 0      ' Input bit to be received
               sD |= m
            waitus(2)
            pinh(SCKpin)               ' Toggle clock pulse
            waitus(2)
            pinl(SCKpin)
            m >>= 1                    ' Advance mask towards LSB
         byte[rA+i] := sD
   pinl(SRCSpin)
   waitus(2)
   pinf(SRCSpin)                       ' Release the various I/O pins
   pinf(SCKpin)
   
PUB outputChr(c)                       ' Display on proper device
   repeat
      if outBuffPtr - @outBuff == lineLen
         abort string("Output buffer overflow")
      byte[outBuffPtr++] := c & $FF
   until (c >>= 8) == 0

PUB outputStr(s)                       ' Display string at s
   repeat strsize(s)
      outputChr(byte[s++])

PUB outputDec(v,d) : width | i,x,z,p   ' Output to buffer
   if d > 10
      d := 0                           ' Allow for 0 to 10 decimal places
   width := 0                          '  with the default being 0
   z := 0
   x := v == NEGX                      ' Check for max negative
   if v < 0
      v := ||(v+x)                     ' Adjust as needed
      outputChr("-")                   '  and output sign
      width++
   i := 1_000_000_000                  ' Initialize divisor
   repeat p from 10 to 1               ' Loop for 10 digits
      if p == d                        ' Insert decimal point
         outputChr(".")
         width++
         z++
      if v >= i
         outputChr(v / i + "0" + x*(i == 1)) 'display digit
         width++
         v //= i
         z++                           ' Flag non-zero found
      elseif z or i == 1
         outputChr("0")               ' Display at least zero
         width++
      i /= 10

PUB outputHex(v,w)                    ' Display hexadecimal
   if w == 0 or w > 8
      w := 8                           ' Default is 8 digits
   v <<= (8 - w) << 2
   repeat w
      outputChr(hex[(v <-= 4) & $F])

PUB outputBin(v,w)                    ' Display binary
   if w == 0 or w > 32
      w := 32                          ' Default is 32 digits
   v <<= 32 - w
   repeat w
      outputChr((v <-= 1) & 1 + "0")

'' outputFormat and outputItem implement a Unix-like printf functionality
'' for the serial console and the mass storage (SD card or flash) file.
'' The outputChr routine is the only link to these files using a device #.
'' First outputFormat is called with a string address.  This string contains
'' Unix-like formatting commands and escape sequences.  The string contents
'' are scanned with characters and escape sequences output to outputChr until
'' a % or zero byte is found.  After this, outputItem is called for each %
'' in the format string.  The argument passed to outputItem is interpreted
'' according to the character(s) in the format string after the %.  The
'' remaining characters until a % or zero byte are also copied to the output
'' and the whole process continues again.
''
'' %x or %nx or %nnx - Convert the value to a hex string of width n, default 8
'' %b or %nb or %nnb - Convert the value to a bit string of width n, default 32
'' %d or %nd or %nnd - Convert the value to a signed decimal string, n decimals
''                     Default is 0 with the decimal point not added in that case
'' %s                - Copy the string whose address is the value
'' %c                - Output the character(s) LSB-MSB, stopping at 0
'' %%                - Output the character %
''
'' \\ - Equivalent to the character "\"
'' \n - A newline character (CR, LF, or CR/LF) determined by NLout variable
'' \l - A linefeed character (LF)
'' \r - A carriage return character (CR)
'' \b - A backspace character (BS) determined by BSout variable
'' \0 - A zero byte
'' \q - A quote character ('"')

PUB outputFormat(s) : result | c, d    ' Start processing format string
   if s >= 0
      outBuffPtr := @outBuff           ' Reset output buffer pointer
      if s == 0
         return false                  ' Only reset output buffer
      outputPtr := s                   ' Reset format string pointer
   repeat while c := byte[outputPtr++] ' Otherwise, continue outputItem
      if c == "%"                      ' Peek ahead to see what to do
         if byte[outputPtr] == "%"
            outputPtr++                ' Copy "%" to output
            outputChr("%")
            next
         return true
      if c == "\"                      ' Special control codes
         c := byte[outputPtr++]
         case c
            "\":   outputChr("\")
            "n":   outputChr(NLout)
            "l":   outputChr(LF)
            "r":   outputChr(CR)
            "b":   outputChr(BSout)
            "q":   outputChr(quote)
            "0":   outputChr(0)
            other: outputPtr--         ' Re-process zero byte if called again
                   return false        ' Return false if no more items or error
         next
      outputChr(c)                     ' Copy others to output
   return false                        ' Return false if no more items

PUB outputItem(i) : result | c, d      ' Interpret format string
   c := byte[outputPtr++]
   if c >= "0" and c <= "9"            ' There may be a single digit after "%"
      d := c - "0"
      c := byte[outputPtr++]
      if c >= "0" and c <= "9"         ' There may be a 2nd digit after the 1st
         d := (d * 10) + (c - "0")
         c := byte[outputPtr++]
   else
      d := 0
   case c
      "x":   outputHex(i,d)            ' %x - item is a hexadecimal value
      "b":   outputBin(i,d)            ' %b - item is a binary value
      "d":   outputDec(i,d)            ' %d - item is a decimal value
      "s":   outputStr(i)              ' %s - item is a string address
      "c":   outputChr(i)              ' %c - item is a character value
      other: return false              ' Return false if no more items
   result := outputFormat(-1)          ' Process everything up to "%" or 0

PUB outputFlush(port) : result         ' Flush buffer to specified serial port
   byte[outBuffPtr] := 0
   if (outBuffPtr - @outBuff) > 0      ' Anything to flush?
      case port
         WiFi: ser.str(port,@outBuff)
         OLED: dsp.str(@outBuff)
               dsp.updateDisplay
   result := outputFormat(0)           ' Reset output pointer
